require(xlsx)
require(stringr)

renameColumns <- function(data, oldNames, newNames)
{
  for(i in 1:length(oldNames))
  {
    names(data)[names(data) == oldNames[i]] <- newNames[i];
  }
  return(data);
} 

postProcessData <- function(data)
{
  data$typeFactor = character(dim(data)[1]); # factor( levels = c("HandPicked", "Generated", "Random"));
  data$typeFactor[data$questType > 0] = "HandPicked";
  data$typeFactor[data$questType == 0] = "Generated";
  data$typeFactor[data$questType < 0] = "Random";
  data$typeFactor = factor(data$typeFactor);
  
  data$numBlocks = str_length(data$questVector)
  data$numBlocks[data$questType > 0] = 3; #all quests of length two were broken (no length 2 quest was generated in the 1st evaluation round)
  
  data$questLikeScale = as.numeric(data$questLikeScale);
  data$questClarityScale = as.numeric(data$questClarityScale);
  data$questUnderstandingScale = as.numeric(data$questUnderstandingScale);
  data$questSimilarityScale = as.numeric(data$questSimilarityScale);
  data$questId = as.numeric(data$questId);
  
#  print(paste("No. of quests length 2:", sum(data$numBlocks != 3)))
  data = subset(data, data$numBlocks == 3);
#  print(paste("No. of hacked quests:", sum(data$questEndState == "Hacked")));
#  print(paste("No. of killed in quest:", sum(data$questEndState == "Killed")));
#  data = subset(data, data$questEndState == "Finished");
  return(data);
  
}

readThesisData <- function()
{
  data = read.xlsx("thesisEvaluation.xlsx", 1)  
  data = renameColumns(data, c("player","questNumer"), c("playerId", "questId"));  
  data$source = "1st";
  data = postProcessData(data);
  return (data);
}

readEntryData <- function(dir)
{
  data = data.frame();
  files = list.files(dir, "*_entry_questionnaire.csv");  
  for(f in files)
  {
    matches = str_match(f,"([0-9]*)_entry_questionnaire\\.csv");
    if(!is.na(matches[1,1]))
    {
      row = suppressWarnings(read.csv(paste(dir,f, sep="/"), stringsAsFactors = FALSE, colClasses="character"));
      row$playerId = matches[2];
      if(length(data) != 0)
      {      
        data = rbind(data, row);
      }
      else
      {
        data = row;
      }
    }
  }
  

  return(data);
}

readQuestLog <- function(dir)
{
  data = data.frame();
  files = list.files(dir, "*.csv");  
  for(f in files)
  {
    matches = str_match(f,"([0-9]*)_questLog_[0-9]\\.csv");
    if(!is.na(matches[1,1]))
    {
      row = suppressWarnings(read.csv(paste(dir,f, sep="/"), stringsAsFactors = FALSE, colClasses="character"));
      if(length(data) != 0)
      {      
        data = rbind(data, row);
      }
      else
      {
        data = row;
      }
    }
  }
  
  data = renameColumns(data, c("player","questNumer"), c("playerId", "questId"));
  
  return(data);  
}

readQuestionnaires <- function(dir)
{
  data = data.frame();
  files = list.files(dir, "*.csv");  
  for(f in files)
  {
    matches = str_match(f,"([0-9]*)_quest_questionnaire_([0-9])\\.csv");
    if(!is.na(matches[1,1]))
    {
      row = suppressWarnings(read.csv(paste(dir,f, sep="/"), stringsAsFactors = FALSE, colClasses="character"));
      if(sum(names(row) == "questSimilarityScale") == 0) #similarity does not exist - first questionnaire
      {
        row$questSimilarityScale = -1;
      }
      row$playerId = matches[2];
      row$questId = matches[3];
      if(length(data) != 0)
      {      
        data = rbind(data, row);
      }
      else
      {
        data = row;
      }
    }
  }
  
  
  return(data);  
}



readSecondData <- function()
{
  dir = "secondEvalAllProcessed";
  entryData = readEntryData(dir);
  questLog = readQuestLog(dir);
  questionnaires = readQuestionnaires(dir);
  
  quests = merge(questLog, questionnaires, by = c("playerId","questId"));
  data = merge(entryData, quests, by = c("playerId") );
  
  data$source = "2nd";
  
  data = postProcessData(data);
  return (data);
}

thesisData = readThesisData();
secondData = readSecondData();
allData = rbind(thesisData,secondData);

allData$isHP = allData$typeFactor == "HandPicked"
allData$notHP = allData$typeFactor != "HandPicked"
hpToNonHP = merge(allData, allData, by.x = c("playerId", "isHP"), by.y = c("playerId", "notHP"))
hpToNonHP = subset(hpToNonHP, hpToNonHP$isHP);
aggregate(hpToNonHP$questLikeScale.x <= hpToNonHP$questLikeScale.y, by = list(hpToNonHP$questType.y), FUN = mean)
aggregate(hpToNonHP$questLikeScale.x < hpToNonHP$questLikeScale.y, by = list(hpToNonHP$questType.y), FUN = mean)
